"""
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
P3D script : convertSimpleToAdvancedChannel
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Script Target : 
|~----------------------------------------------------------------------------~|

 This script affects : animation channels

|~----------------------------------------------------------------------------~|

Compatible Versions : 
|~----------------------------------------------------------------------------~|

 This script is compatible with versions : 2022.0

|~----------------------------------------------------------------------------~|


Requirements : 
|~----------------------------------------------------------------------------~|

 Must have .P3D project (CurrentP3DSession)

 Special conditions : None

|~----------------------------------------------------------------------------~|



Description :
|~----------------------------------------------------------------------------~|

 This script in its first version will convert all channels to complex channel.
 In its second version it will only convert the selected channel to advance
 And in its final version a ui will allow you to choose in between the first two.


|~----------------------------------------------------------------------------~|


Usage:
|~----------------------------------------------------------------------------~|
 
 Classic use case in P3D :  Scripting/Execute Script -> choose script

|~----------------------------------------------------------------------------~|

Data output format :
|~----------------------------------------------------------------------------~|

 A console message will be presented indicating the number of channels converted.

|~----------------------------------------------------------------------------~|

"""
from math import radians

from p3dsdk import AnimationEasingType, SceneObjectType, KeyFrameTangentType, Vector2, Database, \
    ChannelsCurveAnimation, ChannelsSimpleAnimation
from p3dsdk import CurrentP3DSession

TANGENT_VALUE_EASING_TYPE = {
    AnimationEasingType.LINEAR: (Vector2(0, 0), Vector2(0, 0)),
    AnimationEasingType.EASING_IN: (Vector2(1, 0), Vector2(0, -1)),
    AnimationEasingType.EASING_OUT: (Vector2(0, 1), Vector2(-1, 0)),
    AnimationEasingType.EASING_IN_OUT: (Vector2(1, 0), Vector2(-1, 0)),
}

TANGENT_TYPE_FROM_EASING_TYPE = {
    AnimationEasingType.LINEAR: KeyFrameTangentType.LINEAR,
    AnimationEasingType.EASING_IN: KeyFrameTangentType.BEZIER,
    AnimationEasingType.EASING_OUT: KeyFrameTangentType.BEZIER,
    AnimationEasingType.EASING_IN_OUT: KeyFrameTangentType.BEZIER
}

ANIMATION_CHANNEL_TYPE_FROM_KINEMATIC_OBJECT = {
    SceneObjectType.NULL_OBJECT: lambda new_ch_curve, kine, channel_name: new_ch_curve.add_null_channel_animation(kine,
                                                                                                                  channel_name),
    SceneObjectType.AXIS: lambda new_ch_curve, kine, channel_name: new_ch_curve.add_axis_channel_animation(kine),
    SceneObjectType.VECTOR: lambda new_ch_curve, kine, channel_name: new_ch_curve.add_vector_channel_animation(kine)
}


def create_curve_channel_from_simple_channel(data: Database,
                                             simple_channel: ChannelsSimpleAnimation) -> ChannelsCurveAnimation:
    new_ch_curve = None
    model = data.list_models()[0]
    for kine in model.list_kinematic_objects(True):
        if len(simple_channel.animated_channels_for(kine.id)) > 0:
            new_ch_curve = data.create_channels_curve_animation(
                simple_channel.name + " vCurve") if new_ch_curve is None else new_ch_curve

            for channel_name in simple_channel.animated_channels_for(kine.id):
                ANIMATION_CHANNEL_TYPE_FROM_KINEMATIC_OBJECT[kine.type](new_ch_curve, kine, channel_name)

            new_animation = new_ch_curve.list_curves()[0]
            initial_value = simple_channel.initial_value
            final_value = simple_channel.final_value

            if kine.type == SceneObjectType.AXIS:
                initial_value = radians(initial_value)
                final_value = radians(final_value)

            first = new_animation.insert_key_frame(0.1, initial_value)
            final = new_animation.insert_key_frame(simple_channel.duration, final_value)

            first.right_tangent_type = TANGENT_TYPE_FROM_EASING_TYPE[simple_channel.easing]
            final.left_tangent_type = TANGENT_TYPE_FROM_EASING_TYPE[simple_channel.easing]

            first.right_tangent_value, final.left_tangent_value = TANGENT_VALUE_EASING_TYPE[simple_channel.easing]

    return new_ch_curve


def create_single_curve_channel_from_simple_channel(database: Database, channel_name: str):
    """Create an advanced curve channel from the given simple channel animation"""
    for channel in database.list_channels_simple_animations():
        if channel.name == channel_name:
            create_curve_channel_from_simple_channel(database, channel)
            return


def create_curve_channel_from_simple_channel_all(database: Database):
    """Convert all the simple channel animation in the database to advanced curve channel animation"""
    for channel in database.list_channels_simple_animations():
        create_curve_channel_from_simple_channel(database, channel)


def check_preconditions(db: Database):
    if len(db.list_models()) == 0:
        raise Exception('No model to use!')
    if len(db.list_channels_simple_animations()) == 0:
        raise Exception('No simple animation in the database to convert.')


if __name__ == "__main__":
    with CurrentP3DSession(33900) as p3d:
        database = p3d.data
        try:
            check_preconditions(database)
            create_curve_channel_from_simple_channel_all(database)
            print("A curves animation channel version has been created for each simple animation channel.")

        except Exception as e:
            print(str(e))
